@@ -40,7 +40,6 @@ import time
 import zipfile
 from platform import system
 
-import create_aot
 from common import check_python_version
 from common import compare_sha256
 from common import download_https_file
@@ -48,7 +47,9 @@ from common import fetch_https_binary_file
 from common import handle_ctrl_c
 from common import make_sure_path_exists
 from common import print_banner
-from common import write_sha256_file
+
+from create_aot import get_flutter_sdk_version
+from create_aot import create_platform_aot
 
 
 def main():
@@ -85,13 +86,29 @@ def main():
     parser.add_argument('--plugin-platform', default='linux', type=str, help='specify plugin platform type')
     parser.add_argument('--create-aot', default=False, action='store_true', help='Generate AOT')
     parser.add_argument('--app-path', default='', type=str, help='Specify Application path')
+    parser.add_argument('--arch', default=get_flutter_arch(), type=str, help='specify flutter architecture')
+    parser.add_argument('--copy-dconf-user', default=False, action='store_true', help='copy $HOME/.confi/dconf/user to $FLUTTER_WORKSPACE')
 
     args = parser.parse_args()
 
+    #
+    # Generate Release/Profile AOT
+    #
     if args.create_aot:
         if args.app_path == '':
             sys.exit("Must specify value for --app-path")
 
+        set_gen_snapshot('release', args.arch)
+        create_platform_aot(args.app_path, get_flutter_sdk_version())
+        return
+
+    #
+    # Copy dconf user to workspace
+    #
+    if args.copy_dconf_user:
+        copy_dconf_user()
+        return
+
     #
     # Find GIT Commit where flutter analyze returns true
     #
@@ -122,6 +139,12 @@ def main():
 
     print_banner("Setting up Flutter Workspace in: %s" % workspace)
 
+    #
+    # Recursively change ownership to logged in user
+    #
+    user = get_process_stdout('logname').split('\n')
+    cmd = ['sudo', 'chown', '-R', f'{user[0]}:{user[0]}', workspace]
+
     #
     # Install minimum package
     #
@@ -134,11 +157,6 @@ def main():
     subprocess.check_call([sys.executable, '-m', 'venv', venv_dir], stdout=subprocess.DEVNULL)
     os.environ['PATH'] = '%s:%s' % (os.path.join(venv_dir, 'bin'), os.environ.get('PATH'))
 
-    #
-    # Install required python modules
-    #
-    subprocess.check_call(['pip3', 'install', 'pycurl', 'toml', 'python-dotenv'], stdout=subprocess.DEVNULL)
-
     #
     # Control+C handler
     #
@@ -159,7 +177,7 @@ def main():
     #
     if args.fetch_engine:
         print_banner("Fetching Engine Artifacts")
-        get_flutter_engine_runtime(True)
+        get_flutter_engine_runtime(True, args.arch)
         return
 
     #
@@ -204,16 +222,6 @@ def main():
 
         clear_folder(vscode_folder)
 
-    #
-    # Generate Release/Profile AOT
-    #
-    if args.create_aot:
-        if args.app_path != '':
-            create_aot.create_platform_aot(args.app_path)
-        else:
-            sys.exit("Must specify value for --app-path")
-        return
-
     #
     # Fast Boot
     #
@@ -293,7 +301,7 @@ def main():
     #
     # Flutter Engine Runtime
     #
-    get_flutter_engine_runtime(clean_workspace)
+    get_flutter_engine_runtime(clean_workspace, args.arch)
 
     #
     # Create environmental setup script
@@ -342,12 +350,30 @@ def clear_folder(dir_):
         shutil.rmtree(dir_)
 
 
+def copy_dconf_user():
+    """ Copies $HOME/.config/dconf/user to workspace """
+    import shutil
+    from pathlib import Path
+
+    workspace = Path(os.environ.get('FLUTTER_WORKSPACE'))
+    dconf_dst = workspace.joinpath('.config', 'flutter', 'dconf')
+    make_sure_path_exists(dconf_dst)
+
+    workspace = Path(os.environ.get('FLUTTER_WORKSPACE'))
+    dconf_user_src = os.path.join(os.environ.get('HOME'), '.config', 'dconf', 'user')
+    dconf_user_dst = workspace.joinpath('.config', 'flutter', 'dconf', 'user')
+
+    print(f'Copying {dconf_user_src} to {dconf_user_dst}')
+    shutil.copy(dconf_user_src, dconf_user_dst)
+    print_banner('Copied')
+
+
 def get_workspace_config(path):
     """ Returns workspace config """
 
-    if os.path.isdir(path):
+    data = {'globals': None, 'repos': None, 'platforms': []}
 
-        data = {'globals': None, 'repos': None, 'platforms': []}
+    if os.path.isdir(path):
 
         import glob
         for filename in sorted(glob.glob(os.path.join(path, '*.json'))):
@@ -555,47 +581,61 @@ def get_repo(base_folder, uri, branch, rev):
     repo_name = repo_name.split(".")
     repo_name = repo_name[0]
 
+    print_banner(f'Fetching: {repo_name}')
+
     git_folder = os.path.join(base_folder, repo_name)
+    git_hidden_folder = os.path.join(git_folder, '.git')
 
-    git_folder_git = os.path.join(base_folder, repo_name, '.git')
-
-    is_exist = os.path.exists(git_folder_git)
-    if is_exist:
-
+    # print_banner(f'Checking if file exists: {git_hidden_folder}')
+    if os.path.exists(git_hidden_folder):
+        # print_banner(f'git reset --hard: {repo_name}')
         cmd = ['git', 'reset', '--hard']
         subprocess.check_call(cmd, cwd=git_folder)
 
+        # print_banner(f'git fetch --all: {repo_name}')
         cmd = ['git', 'fetch', '--all']
         subprocess.check_call(cmd, cwd=git_folder)
 
-        cmd = ['git', 'checkout', branch]
+        # print_banner(f'git pull: {repo_name}')
+        cmd = ['git', 'pull', 'origin', branch]
         subprocess.check_call(cmd, cwd=git_folder)
-
-        cmd = ['git', 'pull']
-        subprocess.check_call(cmd, cwd=git_folder)
-
     else:
+        # print_banner(f'Checking if folder exists: {git_folder}')
+        if (os.path.exists(git_folder)):
+            # print_banner(f'rm -rf {git_folder} ||true')
+            subprocess.run(['rm','-rf', git_folder, '||','true'], cwd=base_folder)
 
+        # print_banner(f'git clone {uri} -b {branch} {repo_name}')
         cmd = ['git', 'clone', uri, '-b', branch, repo_name]
         subprocess.check_call(cmd, cwd=base_folder)
 
     if rev:
-
-        cmd = ['git', 'reset', '--hard', rev]
+        # print_banner(f'git checkout {rev}')
+        cmd = ['git', 'checkout', rev]
+        subprocess.check_call(cmd, cwd=git_folder)
+    else:
+        # print_banner(f'git checkout {branch}')
+        cmd = ['git', 'checkout', branch]
         subprocess.check_call(cmd, cwd=git_folder)
 
     # get lfs
     git_lfs_file = os.path.join(base_folder, repo_name, '.gitattributes')
+    # print_banner(f'Checking if folder exists: {git_lfs_file}')
     if os.path.exists(git_lfs_file):
+        # print_banner(f'Fetching LFS: {repo_name}')
         cmd = ['git', 'lfs', 'fetch', '--all']
         subprocess.check_call(cmd, cwd=git_folder)
 
     # get all submodules
     git_submodule_file = os.path.join(base_folder, repo_name, '.gitmodules')
+    # print_banner(f'Checking if folder exists: {git_submodule_file}')
     if os.path.exists(git_submodule_file):
+        # print_banner(f'Fetching submodules: {repo_name}')
         cmd = ['git', 'submodule', 'update', '--init', '--recursive']
         subprocess.check_call(cmd, cwd=git_folder)
 
+    print_banner(f'Fetched: {repo_name}')
+
 
 def get_workspace_repos(base_folder, config):
     """ Clone GIT repos referenced in config repos dict to base_folder """
@@ -1070,25 +1110,58 @@ def get_host_machine_arch():
     return platform.machine()
 
 
-def get_google_flutter_engine_url():
+def get_flutter_engine_commit():
     workspace = os.environ.get('FLUTTER_WORKSPACE')
     if not workspace:
         sys.exit("FLUTTER_WORKSPACE not set")
 
     flutter_sdk_path = os.path.join(workspace, 'flutter')
-    arch = get_host_machine_arch()
 
     engine_version = get_flutter_engine_version(flutter_sdk_path)
     os.environ['FLUTTER_ENGINE_VERSION'] = engine_version
-
-    url = f'https://github.com/meta-flutter/flutter-engine/releases/download/linux-engine-sdk-debug-{arch}-{engine_version}/linux-engine-sdk-debug-{arch}-{engine_version}.tar.gz'
-    return url, engine_version
+    return engine_version
 
 
-def get_flutter_engine_runtime(clean_workspace):
+def get_flutter_arch():
+    host_arch = get_host_machine_arch()
+    if host_arch == 'x86_64':
+        return 'x64'
+    elif host_arch == 'arm64':
+        return 'arm64'
+
+
+def set_gen_snapshot(runtime, arch):
+    engine_sdk = f'engine-sdk-{runtime}-{arch}'
+    linux_runtime = f'linux_{runtime}_x64'
+    commit = get_flutter_engine_commit()
+    platform_path = get_platform_working_dir('flutter-engine')
+
+    engine_sdk_root = os.path.join(platform_path, commit, engine_sdk, 'src', 'out', linux_runtime, 'engine-sdk')
+
+    gen_snapshot = os.path.join(engine_sdk_root, 'bin', 'gen_snapshot')
+    if not os.path.exists:
+        get_flutter_engine_artifacts(True, runtime)
+
+    if not os.path.exists:
+        sys.exit('engine-sdk error')
+
+    os.environ['GEN_SNAPSHOT'] = gen_snapshot
+
+
+def get_engine_sdk_url(runtime, arch):
+    commit = get_flutter_engine_commit()
+    if arch == 'x64':
+        arch = 'x86_64'
+    elif arch == 'arm':
+        arch = 'armv7hf'
+    url = f'https://github.com/meta-flutter/flutter-engine/releases/download/linux-engine-sdk-{runtime}-{arch}-{commit}/linux-engine-sdk-{runtime}-{arch}-{commit}.tar.gz'
+    return url, commit
+
+
+def get_flutter_engine_artifacts(clean_workspace, runtime, arch):
     """Downloads Flutter Engine Runtime"""
 
-    base_url, engine_version = get_google_flutter_engine_url()
+    base_url, engine_version = get_engine_sdk_url(runtime, arch)
 
     _, filename = os.path.split(base_url)
 
@@ -1099,7 +1172,7 @@ def get_flutter_engine_runtime(clean_workspace):
     archive_file = os.path.join(cwd_engine, filename)
     sha256_file = os.path.join(cwd_engine, filename + '.sha256')
 
-    bundle_folder = os.path.join(cwd, 'bundle')
+    bundle_folder = os.path.join(cwd, f'bundle-{runtime}-{arch}')
     os.environ['BUNDLE_FOLDER'] = bundle_folder
 
     if not compare_sha256(archive_file, sha256_file):
@@ -1112,29 +1185,34 @@ def get_flutter_engine_runtime(clean_workspace):
     else:
         print_banner("Skipping Engine artifact download")
 
-    if clean_workspace:
-        if os.path.exists(bundle_folder):
-            cmd = ["rm", "-rf", bundle_folder]
-            subprocess.check_output(cmd, cwd=cwd)
-
-    restore_folder = os.path.join(cwd_engine, 'engine-sdk')
+    restore_folder = os.path.join(cwd_engine, f'engine-sdk-{runtime}-{arch}')
     make_sure_path_exists(restore_folder)
     subprocess.check_call(['tar', '-xzf', archive_file, '-C', restore_folder])
 
+    if clean_workspace:
+        if os.path.exists(bundle_folder):
+            subprocess.check_output(["rm", "-rf", bundle_folder], cwd=cwd)
+
     data_folder = os.path.join(bundle_folder, 'data')
     make_sure_path_exists(data_folder)
 
-    icudtl_src = os.path.join(restore_folder, 'src', 'out', 'linux_debug_x64', 'engine-sdk', 'data', 'icudtl.dat')
+    icudtl_src = os.path.join(restore_folder, 'src', 'out', f'linux_{runtime}_{arch}', 'engine-sdk', 'data', 'icudtl.dat')
 
     lib_folder = os.path.join(bundle_folder, 'lib')
     make_sure_path_exists(lib_folder)
 
-    libflutter_engine_src = os.path.join(restore_folder, 'src', 'out', 'linux_debug_x64', 'engine-sdk', 'lib', 'libflutter_engine.so')
+    libflutter_engine_src = os.path.join(restore_folder, 'src', 'out', f'linux_{runtime}_{arch}', 'engine-sdk', 'lib', 'libflutter_engine.so')
 
     subprocess.check_call(["cp", icudtl_src, f'{data_folder}'])
     subprocess.check_call(["cp", libflutter_engine_src, f'{lib_folder}'])
 
 
+def get_flutter_engine_runtime(clean_workspace, arch):
+    get_flutter_engine_artifacts(clean_workspace, 'release', arch)
+    get_flutter_engine_artifacts(clean_workspace, 'profile', arch)
+    get_flutter_engine_artifacts(clean_workspace, 'debug', arch)
+
+
 def handle_conditionals(conditionals, cwd):
     if not conditionals:
         return
@@ -1244,16 +1322,17 @@ def handle_http_obj(obj, host_machine_arch, cwd, cookie_file, netrc):
                 local_url = artifact.get('url')
                 if local_url is None:
                     local_url = url
+
                 base_url = local_url + artifact['endpoint']
                 base_url = os.path.expandvars(base_url)
                 filename = get_filename_from_url(base_url)
 
-                print(base_url)
-                print(filename)
+                print(f'url: {base_url}')
+                print(f'filename: {filename}')
 
                 futures.append(executor.submit(download_https_file, cwd, base_url, filename, cookie_file,
                                                netrc, artifact.get('md5'), artifact.get('sha1'),
-                                               artifact.get('sha256')))
+                                               artifact.get('sha256'), True))
                 subprocess.check_call(
                     ['sudo', '-v'], stdout=subprocess.DEVNULL)
 
@@ -1570,7 +1649,7 @@ def get_platform_working_dir(platform_id):
     cwd = workspace.joinpath('.config', 'flutter_workspace', platform_id)
     os.environ["PLATFORM_ID_DIR_RELATIVE"] = '.' + platform_id
     os.environ["PLATFORM_ID_DIR"] = str(cwd)
-    print("Working Directory: %s" % cwd)
+    print(f'Working Directory: {cwd}')
     make_sure_path_exists(cwd)
     return cwd
 
@@ -1920,14 +1999,14 @@ def install_minimum_runtime_deps():
         if os_release_id == 'ubuntu':
             cmd = ['sudo', 'apt', 'update', '-y']
             subprocess.check_output(cmd)
-            packages = 'git git-lfs curl libcurl4-openssl-dev libssl-dev libgtk-3-dev'.split(' ')
+            packages = 'git git-lfs curl libcurl4-openssl-dev libssl-dev libgtk-3-dev python3.8-venv python3-pycurl python3-toml python3-dotenv'.split(' ')
             for package in packages:
                 ubuntu_install_pkg_if_not_installed(package)
 
         elif os_release_id == 'fedora':
             cmd = ['sudo', 'dnf', '-y', 'update']
             subprocess.check_output(cmd)
-            packages = 'dnf-plugins-core git git-lfs curl libcurl-devel openssl-devel gtk3-devel'.split(' ')
+            packages = 'dnf-plugins-core git git-lfs curl libcurl-devel openssl-devel gtk3-devel python3-virtualenv python3-pycurl python3-toml python3-dotenv'.split(' ')
             for package in packages:
                 fedora_install_pkg_if_not_installed(package)
 
